@@ -6,7 +6,7 @@ from turbogears.widgets.base import Widget, CompoundWidget, WidgetsList, \
 
 try:
     set
-except NameError:
+except NameError: # Python 2.3
     from sets import Set as set
 
 __all__ = ["InputWidget", "CompoundInputWidget", "RepeatingInputWidget",
@@ -22,6 +22,7 @@ __all__ = ["InputWidget", "CompoundInputWidget", "RepeatingInputWidget",
 #############################################################################
 # Functions and classes to manage input widgets                             #
 #############################################################################
+
 def append_to_path(widget, repetition):
     path = []
     if request_available():
@@ -29,7 +30,7 @@ def append_to_path(widget, repetition):
             path = request.tg_widgets_path
         else:
             request.tg_widgets_path = path
-    if (not path) or (path[-1].widget is not widget):
+    if not path or path[-1].widget is not widget:
         path.append(Bunch(widget=widget,
                           repetition=repetition))
         return True
@@ -64,10 +65,10 @@ def path_from_item(item, base_path=None):
         index = 0
         for key in path:
             if '-' in key:
-                a, b = key.split('-')
-                path[index] = (a, int(b))
+                key, num = key.split('-')
+                path[index] = key, int(num)
             else:
-                path[index] = (key, None)
+                path[index] = key, None
             index += 1
     elif hasattr(item, "name"):
         path = [(item.name, None)]
@@ -130,7 +131,7 @@ def build_name_from_path(path, repeating_marker='-', nesting_marker='.'):
     return nesting_marker.join(name)
 
 ###############################################################################
-# Base class for a widget that can generate input fort the app.
+# Base class for a widget that can generate input for the app.
 ###############################################################################
 
 class InputWidget(Widget):
@@ -156,7 +157,9 @@ class InputWidget(Widget):
         if name is not None and ('-' in name or '.' in name):
             raise ValueError, ("The name of an InputWidget must not contain "
                                "the '-' or '.' characters")
+
         super(InputWidget, self).__init__(name, **params)
+
         if validator:
             self.validator = validator
 
@@ -175,11 +178,7 @@ class InputWidget(Widget):
     def _get_is_validated(self):
         if self.path:
             validated_form = getattr(request, "validated_form", None)
-            form = self.path[0].widget
-            if form is validated_form:
-                return True
-            else:
-                return False
+            return self.path[0].widget is validated_form
         else:
             return False
     is_validated = property(_get_is_validated)
@@ -228,6 +227,8 @@ class InputWidget(Widget):
         else:
             if value is None:
                 value = self.default
+                if callable(value):
+                    value = value()
             if self.validator and params["convert"] and not input_submitted:
                 value = self.validator.from_python(value)
         return value
@@ -254,6 +255,7 @@ class InputWidget(Widget):
         d["error"] = retrieve_value_by_path(errors, self.name_path)
 
 class CompoundInputWidget(CompoundWidget, InputWidget):
+
     def update_params(self, params):
         super(CompoundInputWidget, self).update_params(params)
         params['error_for'] = lambda f: self.error_for(f, True)
@@ -311,6 +313,7 @@ class CompoundInputWidget(CompoundWidget, InputWidget):
         return super(CompoundWidget, self).adjust_value(value, **params)
 
 class RepeatingRange(object):
+
     def __init__(self, repetitions, bunch):
         self.__sequence = repetitions
         self.__next_value = 0
@@ -331,6 +334,7 @@ class RepeatingRange(object):
 
 class RepeatingInputWidget(CompoundInputWidget):
     """Base class for a compound widget which can be repeated."""
+
     repeating = True
     params = ["repetitions"]
     params_doc = {'repetitions': 'Number of repetitions that should be '
@@ -391,6 +395,7 @@ class RepeatingInputWidget(CompoundInputWidget):
 #############################################################################
 # Base classes                                                              #
 #############################################################################
+
 class FormField(InputWidget):
     """An input widget that can be included inside a Form or Fieldset.
 
@@ -402,7 +407,9 @@ class FormField(InputWidget):
     help_text
        The help text that the container form/fieldset will generate for the
        widget.
+
     """
+
     _name = "widget"
     label = None
     help_text = None
@@ -414,6 +421,7 @@ class FormField(InputWidget):
 
     def _get_name(self):
         return self._name
+
     def _set_name(self, name):
         self._name = name
         if self.label is None:
@@ -441,8 +449,10 @@ class FormField(InputWidget):
         super(FormField, self).__init__(name, **kw)
         if label is not None:
             self.label = label
+
         if help_text is not None:
             self.help_text = help_text
+
         if self.field_class is None:
             self.field_class = self.__class__.__name__.lower()
 
@@ -498,6 +508,7 @@ class FormFieldsContainer(CompoundInputWidget):
        - error_for(name)
            Returns the validation error the child named ``name`` generated.
            Again, ``name`` can also be a widget instance.
+
     """
 
     member_widgets = ["fields", "hidden_fields"]
@@ -568,14 +579,16 @@ class RepeatingFormField(RepeatingInputWidget, CompoundFormField):
 #############################################################################
 # Fields                                                                    #
 #############################################################################
+
 class Label(FormField):
-    "A simple label for a form field."
+    """A simple label for a form field."""
 
     template = """
     <label xmlns:py="http://purl.org/kid/ns#"
         id="${field_id}"
         class="${field_class}"
         py:content="value"
+        py:attrs="attrs"
     />
     """
     params = ["attrs"]
@@ -587,7 +600,7 @@ class LabelDesc(CoreWD):
     for_widget = Label(default="Sample Label")
 
 class TextField(FormField):
-    "A standard, single-line text field."
+    """A standard, single-line text field."""
 
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -611,7 +624,7 @@ class TextFieldDesc(CoreWD):
                            attrs=dict(size="30"))
 
 class PasswordField(FormField):
-    "A password field which masks letters with * characters."
+    """A password field which masks letters with * characters."""
 
     template = """
     <input xmlns:py="http://purl.org/kid/ns#"
@@ -736,18 +749,21 @@ class ImageButton(Button):
         src="${src}"
         width="${width}"
         height="${height}"
+        alt="${alt}"
         class="${field_class}"
         value="${value}"
         py:attrs="attrs"
     />
     """
-    params = ["src", "width", "height"]
+    params = ["src", "width", "height", "alt"]
     params_doc = {'src' : 'Source of the image',
                   'width' : 'Width of the image',
-                  'height' : 'Height of the image'}
+                  'height' : 'Height of the image',
+                  'alt': 'Alternate text for the image'}
     src = None
     width = None
     height = None
+    alt = None
 
 class ImageButtonDesc(CoreWD):
     name = "Image Button"
@@ -1133,55 +1149,60 @@ class RepeatingFieldSetDesc(CoreWD):
 #############################################################################
 # Forms                                                                     #
 #############################################################################
+
 class Form(FormFieldsContainer):
     form = True
     name = "form"
     member_widgets = ["submit"]
-    params = ["action", "method", "form_attrs", "submit_text"]
-    params_doc = {'action' : 'Url to POST/GET the form data',
-                  'method' : 'HTTP request method',
-                  'form_attrs' : 'Extra (X)HTML attributes for the form tag',
-                  'submit_text' : 'Text for the submit button',
-                  'disabled_fields' : 'List of names of the fields we want to '
-                                      'disable'}
+    params = ["action", "method", "form_attrs", "use_name", "submit_text"]
+    params_doc = {'action': 'Url to POST/GET the form data',
+        'method': 'HTTP request method',
+        'form_attrs': 'Extra (X)HTML attributes for the form tag',
+        'use_name': 'Whether to use the name instead of the id attribute',
+        'submit_text': 'Text for the submit button',
+        'disabled_fields': 'List of names of the fields we want to disable'}
     submit = SubmitButton()
     action = None
     method = "post"
     form_attrs = {}
+    use_name = True
     submit_text = None
 
     def update_params(self, d):
+        name = d.get('name')
         super(Form, self).update_params(d)
-        # A form is always at the beginning of the path so we just
-        # use its name
-        d["name"] = self.name
+        d['name'] = name or self.name
         if self.file_upload:
-            d["form_attrs"]['enctype'] = "multipart/form-data"
+            d['form_attrs']['enctype'] = 'multipart/form-data'
 
     def validate(self, value, state=None):
         if self.validator:
             return self.validator.to_python(value, state)
 
     def _get_errors(self):
-        return getattr(request, "validation_errors", None)
+        return getattr(request, 'validation_errors', None)
     errors = property(_get_errors)
 
 class TableForm(Form):
+    """Form with simple table layout."""
+
+    # note that even hidden fields must live inside a block element
     template = """
     <form xmlns:py="http://purl.org/kid/ns#"
-        name="${name}"
+        name="${use_name and name or None}"
+        id="${not use_name and name or None}"
         action="${action}"
         method="${method}"
         class="tableform"
         py:attrs="form_attrs"
     >
-        <div py:for="field in hidden_fields"
-            py:replace="field.display(value_for(field), **params_for(field))"
-        />
+        <div py:if="hidden_fields" style="display:none">
+            <div py:for="field in hidden_fields"
+                py:replace="field.display(value_for(field), **params_for(field))"
+            />
+        </div>
         <table border="0" cellspacing="0" cellpadding="2" py:attrs="table_attrs">
-            <tr py:for="i, field in enumerate(fields)"
-                class="${i%2 and 'odd' or 'even'}"
-            >
+            <tr py:for="i, field in enumerate(fields)" class="${i%2 and 'odd' or 'even'}">
                 <th>
                     <label class="fieldlabel" for="${field.field_id}" py:content="field.label" />
                 </th>
@@ -1221,26 +1242,30 @@ class TableFormDesc(CoreWD):
     </div>
     """
 
+    [expose()]
     def save(self, **kw):
         return """Received data from TableForm:<br />%r""" % kw
-    save = expose()(save)
 
 class ListForm(Form):
+    """Form with simple list layout."""
+
+    # note that even hidden fields must live inside a block element
     template = """
     <form xmlns:py="http://purl.org/kid/ns#"
-        name="${name}"
+        name="${use_name and name or None}"
+        id="${not use_name and name or None}"
         action="${action}"
         method="${method}"
         class="listform"
         py:attrs="form_attrs"
     >
-        <div py:for="field in hidden_fields"
-            py:replace="field.display(value_for(field), **params_for(field))"
-        />
+        <div py:if="hidden_fields" style="display:none">
+            <div py:for="field in hidden_fields"
+                py:replace="field.display(value_for(field), **params_for(field))"
+            />
+        </div>
         <ul py:attrs="list_attrs">
-            <li py:for="i, field in enumerate(fields)"
-                class="${i%2 and 'odd' or 'even'}"
-            >
+            <li py:for="i, field in enumerate(fields)" class="${i%2 and 'odd' or 'even'}">
                 <label class="fieldlabel" for="${field.field_id}" py:content="field.label" />
                 <span py:replace="field.display(value_for(field), **params_for(field))" />
                 <span py:if="error_for(field)" class="fielderror" py:content="error_for(field)" />
@@ -1273,6 +1298,6 @@ class ListFormDesc(CoreWD):
     </div>
     """
 
+    [expose()]
     def save(self, **kw):
         return """Received data from ListForm:<br />%r""" % kw
-    save = expose()(save)
